/*
// File:		MessagePartEnumerator.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using Emai;
using Emai.Mime;

namespace Common
{
    class MessagePartEnumerator : Emai.Mime.MessagePartEnumerator
    {
        private UInt32 mCounterKey;
        private String mSubject;
        public String Subject
        {
            get
            {
                return this.mSubject;
            }
        }

        private ContactList mContactFrom;
        public String From
        {
            get
            {
                if (this.mContactFrom != null)
                    return this.GetContactStorage(ref this.mContactFrom);
                return "";
            }
        }

        private ContactList mContactTo;
        public String To
        {
            get
            {
                if (this.mContactTo != null)
                    return this.GetContactStorage(ref this.mContactTo);
                return "";
            }
        }

        private ContactList mContactSender;
        public String Sender
        {
            get
            {
                if (this.mContactSender != null)
                    return this.GetContactStorage(ref this.mContactSender);
                return "";
            }
        }

        private ContactList mContactReplyTo;
        public String ReplyTo
        {
            get
            {
                if (this.mContactReplyTo != null)
                    return this.GetContactStorage(ref this.mContactReplyTo);
                return "";
            }
        }

        private ContactList mContactBCC;
        public String BCC
        {
            get
            {
                if (this.mContactBCC != null)
                    return this.GetContactStorage(ref this.mContactBCC);
                return "";
            }
        }

        private ContactList mContactCC;
        public String CC
        {
            get
            {
                if (this.mContactCC != null)
                    return this.GetContactStorage(ref this.mContactCC);
                return "";
            }
        }

        private String mHtmlPart;
        public String Html
        {
            get
            {
                return this.mHtmlPart;
            }
        }

        public String Text
        {
            get
            {
                String res = "";
                foreach (PartData data in this.mTextParts)
                {
                    if (data.GetType() == typeof(PlainTextPartData))
                    {
                        PlainTextPartData textData = (PlainTextPartData)data;
                        if (textData.Text != null)
                            res += textData.Text.ToString();
                    }
                    else if (data.GetType() == typeof(UnicodeTextPartData))
                    {
                        UnicodeTextPartData textData = (UnicodeTextPartData)data;
                        if (textData.Text != null)
                            res += textData.Text;
                    }
                }
                return res;
            }
        }
        private ArrayList mTextParts;

		private List<MessagePart> mAttachments;
        public Int32 AttachmentsCount
        {
            get
            {
                return this.mAttachments.Count;
            }
        }

        public MessagePartEnumerator(Message message)
            : base(message)
        {
            this.mCounterKey = 777;
            this.mAttachments = new List<MessagePart>();
            this.mTextParts = new ArrayList();
            this.mHtmlPart = "";
        }

        private String GetContactStorage(ref ContactList inContactList)
        {
            String str = "";
            UInt32 count = inContactList.GetCount();
            if (count > 0)
            {
                String name;
                String address;
                for (UInt32 i = 0; i < count; ++i)
                {
                    name = inContactList.GetName(i, (UInt32)EnumNullOptions.NullOptions);
                    address = inContactList.GetAddress(i, (UInt32)EnumNullOptions.NullOptions);
                    if ((name.Length > 0) && (address.Length > 0))
                        str = str + name + " - " + address + "; ";
                    else if ((name.Length > 0) && (address.Length == 0)) str = str + name + "; ";
                    else if ((name.Length == 0) && (address.Length > 0)) str = str + address + "; ";
                }
            }
            return str;
        }

        protected override void OnContacts(ContactList inContactList, UInt32 inOptions)
        {
            switch (inOptions)
            {
                case (UInt32)EnumMessageFieldType.From:
                    this.mContactFrom = inContactList;
                    break;

                case (UInt32)EnumMessageFieldType.To:
                    this.mContactTo = inContactList;
                    break;

                case (UInt32)EnumMessageFieldType.Sender:
                    this.mContactSender = inContactList;
                    break;

                case (UInt32)EnumMessageFieldType.ReplyTo:
                    this.mContactReplyTo = inContactList;
                    break;

                case (UInt32)EnumMessageFieldType.CC:
                    this.mContactCC = inContactList;
                    break;

                case (UInt32)EnumMessageFieldType.BCC:
                    this.mContactBCC = inContactList;
                    break;

                default:
                    break;
            }
        }

        protected override void OnSubject(String inSubject)
        {
            this.mSubject = inSubject;
        }

        protected override void OnPlainTextPart(MessagePart inPart, PlainTextPartData inPartData)
        {
            this.mTextParts.Add(inPartData);
        }

        protected override void OnHtmlTextPart(MessagePart inPart, HtmlTextPartData inPartData)
        {
            ASCIIEncoding enc = new ASCIIEncoding();
            this.mHtmlPart = enc.GetString(inPartData.Html);
        }

        protected override void OnUnicodePlainTextPart(MessagePart inPart, UnicodeTextPartData inPartData)
        {
            this.mTextParts.Add(inPartData);
        }

        protected override void OnDataPart(MessagePart inPart, DataPartData inPartData)
        {
			this.mAttachments.Add(inPart);

			String filename = GetFileName(ref inPartData);
			if (filename.Length == 0)
			{
				//The part doesn't have filename specified, we should generate one later
				//Increase internal counter and save it as user data in the message part
				UInt32 counter = 0;
				Byte[] data = new Byte[1];
				this.mMessage.GetUserData(this.mCounterKey, ref data);
				counter = BitConverter.ToUInt32(data, 0);
				++counter;
				this.mMessage.SetUserData(mCounterKey, BitConverter.GetBytes(counter));
				inPart.SetUserData(this.mCounterKey, BitConverter.GetBytes(counter));
			}
        }

        private void OnPart(ref CompositePartData inPartData)
        {
        }

        protected override void OnCompositePartStart(MessagePart inPart, CompositePartData inPartData)
        {
            OnPart(ref inPartData);
        }

        protected override void OnCompositePartEnd(MessagePart inPart, CompositePartData inPartData)
        {
            OnPart(ref inPartData);
        }

		protected override void PreProcessMessageParts()
		{
			UInt32 counter = 0;
			this.mMessage.SetUserData(this.mCounterKey, BitConverter.GetBytes(counter));
		}

		protected override void PostProcessMessageParts()
		{
			this.mMessage.SetUserData(this.mCounterKey, BitConverter.GetBytes(0));
		}

        public void GetAttachmentAtIndex(Int32 inIndex, ref Byte[] outData, ref String outFileName, ref String outContentID)
        {
            PartData data = null;
			this.mAttachments[inIndex].GetData(ref data);
			DataPartData partData = (DataPartData)data;

			outFileName = GetFileName(ref partData);
			if (outFileName.Length == 0)
				outFileName = GenerateFileName(this.mAttachments[inIndex]);

			if (partData.Data != null)
                outData = partData.Data;
            if (partData.ContentID != null)
                outContentID = partData.ContentID;
        }

        private String GetFileName(ref DataPartData inPartData)
        {
            /*Get the pointer to file name of the message attachment*/
			String filename = null;
			if (inPartData.ContentDisposition != null)
			{
				filename = inPartData.ContentDisposition
					.FindParameterValue("filename", (UInt32)EnumNullOptions.NullOptions);
			}
            if (filename == null)
            {
                ContentField contentType = inPartData.ContentType;
                filename = inPartData.ContentType
                    .FindParameterValue("name", (UInt32)EnumNullOptions.NullOptions);
            }

            return filename;
        }

		private String GenerateFileName(MessagePart inPart)
		{
			Byte[] data = new Byte[1];
			inPart.GetUserData(mCounterKey, ref data);
			UInt32 counter = BitConverter.ToUInt32(data, 0);
			return String.Format("ATT{0:D}", counter);
		}
	}
}
