/*
// File:		ImapFolder.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Diagnostics;
using System.Globalization;

using Emai.Imap;
using Emai.Mime;

namespace Imap
{
    class ImapFolder
    {
        public UInt32 Attributes;
        public StringList CustomAttributes;
        public Char Delimiter;
        public String Name;
        public UInt32 Flags;
        private StringList mCustomFlags;
        public StringList CustomFlags
        {
            get
            {
                return this.mCustomFlags;
            }

            set
            {
                this.mCustomFlags = value;
            }
        }
        public Boolean ReadOnly;
        public UInt32 Exists;
        public UInt32 Recent;
        public UInt32 Unseen;
        public UInt32 PermanentFlags;
        private StringList mPermanentCustomFlags;
        public StringList PermanentCustomFlags
        {
            get
            {
                return this.mPermanentCustomFlags;
            }

            set
            {
                this.mPermanentCustomFlags = value;
            }
        }
        private UInt32 mNextUid;
        public UInt32 NextUid
        {
            get
            {
                return this.mNextUid;
            }

            set
            {
                if (mNextUid != value)
                {
                    /*Fetch new messages*/
                }
                mNextUid = value;
            }
        }
        private UInt32 mUidValidity;
        public UInt32 UidValidity
        {
            get
            {
                return this.mUidValidity;
            }

            set
            {
                if (this.mUidValidity != value)
                {
                    Array.Clear(this.mMessages, 0, this.mMessages.Length);
                }
                this.mUidValidity = value;
            }
        }
        private ImapMessageData[] mMessages;
        private ImapSequenceSet mFoundMessageList;
        public ImapSequenceSet FoundMessageList
        {
            get
            {
                return this.mFoundMessageList;
            }

            set
            {
                this.mFoundMessageList = value;
            }
        }

        private ImapSequenceSet mSelectedForDeletion;
        public ImapSequenceSet SelectedForDeletion
        {
            get
            {
                return this.mSelectedForDeletion;
            }

            set
            {
                this.mSelectedForDeletion = value;
            }
        }

        public Boolean IsMarkedForDeletion(UInt32 inMessageNumber)
        {
            if (mSelectedForDeletion != null)
            {
                if (mSelectedForDeletion.ContainsNumber(GetMessageDataAtIndex(inMessageNumber).Uid))
                    return true;
            }
            return false;
        }

        public void FlushSelectedForDeletion()
        {
            mSelectedForDeletion = null;
        }

        public ImapFolder(ref ListResponse inListResponse)
        {
            this.mMessages = new ImapMessageData[0];
            this.Attributes = inListResponse.Attributes;
            StringList customAttributes = inListResponse.CustomAttributes;
            if (customAttributes != null)
                this.CustomAttributes = new StringList(customAttributes);
            this.Delimiter = inListResponse.Delimiter;
            this.Name = inListResponse.MailboxName;
            this.Flags = 0;
            this.ReadOnly = true;
            this.Exists = 0;
            this.Recent = 0;
            this.Unseen = 0;
            this.PermanentFlags = 0;
            this.mNextUid = 0;
            this.mUidValidity = 0;
        }

        public void AddMessageData(ref FetchResponse inResponse)
        {
            Debug.Assert(inResponse.MessageNumber > 0);

            if (inResponse.MessageNumber > this.mMessages.Length)
                Array.Resize<ImapMessageData>(ref this.mMessages, (Int32)inResponse.MessageNumber);

            if (this.mMessages[inResponse.MessageNumber - 1] == null)
                this.mMessages[inResponse.MessageNumber - 1] = new ImapMessageData();

            this.mMessages[inResponse.MessageNumber - 1].Data = inResponse;
        }

        public ImapMessageData GetMessageDataAtIndex(UInt32 inMessageNumber)
        {
            Debug.Assert(inMessageNumber > 0);
            Debug.Assert(inMessageNumber <= Exists);
            if ((inMessageNumber > 0) && (inMessageNumber <= this.mMessages.Length))
                return (ImapMessageData)this.mMessages[(Int32)inMessageNumber - 1];

            return null;
        }

        private ImapMessageData mCompareData;
        private Boolean CompareData(ImapMessageData inData)
        {
            if (this.mCompareData.Uid == inData.Uid)
                return true;
            return false;
        }

        public ImapMessageData GetMessageDataByUid(UInt32 inUID)
        {
            Debug.Assert((inUID <= ((ImapMessageData)this.mMessages[(Int32)this.mMessages.Length - 1]).Uid)
                && (inUID >= ((ImapMessageData)this.mMessages[0]).Uid));
            mCompareData = new ImapMessageData(inUID);
            Int32 location = Array.FindIndex(this.mMessages, CompareData);
            if (location >= 0)
                return this.mMessages[location];
            return null;
        }

        protected ImapMessageData[] RemoveZeros(ref ImapMessageData[] inArray)
        {
            ImapMessageData[] resArray = new ImapMessageData[0];
            Int32 counter = 1;
            for (Int32 i = 0; i < inArray.Length; i++ )
            {
                if (inArray[i] != null)
                {
                    Array.Resize<ImapMessageData>(ref resArray, counter);
                    resArray[counter - 1] = inArray[i];
                    counter++;
                }
            }
            return resArray;
        }

        public void MessageDeleted(UInt32 inMessageNumber)
        {
            Debug.Assert(inMessageNumber > 0);
            Debug.Assert(inMessageNumber <= Exists);
            Debug.Assert(Exists >= this.mMessages.Length);
            //Check if the message data was downloaded before
            if (inMessageNumber <= this.mMessages.Length)
            {
                Array.Clear(this.mMessages, (Int32)(inMessageNumber - 1), 1);
                this.mMessages = RemoveZeros(ref this.mMessages);
            }
            //Modify number of messages which are currently on server
            Debug.Assert(Exists > 0);
            Exists = Exists - 1;
            Debug.Assert(Exists >= this.mMessages.Length);
        }

        public void MessagesAppended(Message[] inMessages, ref ImapSequenceSet inUidSequenceSet)
        {
        }

    }
}
